function [merged_a, merged_alpha] = IMU_sensor_fusion(filt_data1, filt_data2, filt_data3, filt_data4, num_samplings)
    merged_a = [];
    merged_alpha = [];
    for i = 1:num_samplings
        [a_b_bi, alpha_b_bi] = vector_fusion(filt_data1(i,:), filt_data2(i,:), filt_data3(i,:), filt_data4(i,:));

        merged_a = [merged_a; a_b_bi'];
        merged_alpha = [merged_alpha; alpha_b_bi'];
    end 
end

function [a_b_bi, alpha_b_bi] = vector_fusion(IMU1, IMU2, IMU3, IMU4)
    % IMU*: [time [s], gyrox [deg/s], gyroy [deg/s], gyroz [deg/s], 
    %        accsurge [mm/s^2], accsway [mm/s^2], accyaw [mm/s^2]]
    zeta = [IMU1(5:7), IMU2(5:7), IMU3(5:7), IMU4(5:7)]';
    
    l = [-244, 184, 82;
         -950, -130, 0;
         740, 130, 82;
         450, -160, 82];
    W = [];
    for i = 1:4
        S_l = Smtrx(l(i,:));
        H_l = Hmatrix(l(i,:));
        W = [W; eye(3) -S_l H_l]; 
    end
    
    W_inv = pinv(W);
    
    z = W_inv*zeta;
    
    a_b_bi = [eye(3), zeros(3,9)]*z;
    alpha_b_bi = [zeros(3), eye(3), zeros(3,6)]*z;
end

function H = Hmatrix(l)
%Function that creates H matrix
% Calculates the H matrix according to "Disturbance rejection by
% acceleration feedforward for marine surface vessels" by Kjerstad and
% Skjetne

    H = [0, -l(1), -l(1), l(2), l(3), 0; ...
        -l(2), 0, -l(2), l(1), 0, l(3); ...
        -l(3), -l(3), 0, 0, l(1), l(2)];
end