function DFOobs(optimAlg)
    fileID = fopen('results/observer_params.txt','w');
    fprintf(fileID,'optimtype: %i\n', optimAlg);
    fprintf(fileID,'lambdaP lambdaH wc Ts KpP KpH cost_eta cost_nu cost_nu_dot cost\n');
    fclose(fileID);
    
    gamma = 90;
    wo = 0.68*sqrt(gamma);
    %    lambdaP lambdaH wc   Ts   KpP  KpH
    LB = [0.001, 0.001, 1.1*wo, 50, 500, 10]';
    UB = [0.1, 0.1, 3*wo, 100, 1500, 100]';
    if optimAlg == 1 %particleswarm
        options = optimoptions('particleswarm', 'PlotFcn', 'pswplotbestf', 'UseVectorized',false, 'SwarmSize',10);
        [x,fval,exitflag,output] = particleswarm(@CFobs, 6, LB, UB, options);
        
    elseif optimAlg == 2 %patternsearch
        x_0 = [0.01, 0.01, 2*wo, 80, 1000, 60]';
        options = optimoptions(@patternsearch, 'PlotFcn', 'psplotbestf', 'UseVectorized',false);
        [x,fval,exitflag,output] = patternsearch(@CFobs, x_0, [],[],[],[], LB, UB, options);
        
    elseif optimAlg == 3 %fminsearch
        options = optimset('PlotFcns',@optimplotfval);
        x_0 = [0.01, 0.01, 2*wo, 80, 1000, 60]';
        [x,fval,exitflag,output] = fminsearchbnd(@CFobs, x_0, LB, UB, options);
    end
    
    %Set the best gains in the simulation model
    lambdaP = x(1);
    lambdaH = x(2);
    wc = x(3);
    Ts = x(4);
    KpP = x(5);
    KpH = x(6);

    % filter gains
    K11 = -2*(eye(3)-diag([lambdaP, lambdaP, lambdaH]))*(wc/wo);
    K12 = 2*(eye(3)-diag([lambdaP, lambdaP, lambdaH]))*wo;
    K2 = diag([wc, wc, wc]);
    K4 = diag([KpP, KpP, KpH]);
    K3 = 10*(K4/Ts);
    T_b = Ts/sqrt(gamma);
    T_b_inv = eye(3)*1/(T_b);
    Kw1 = 2*diag([lambdaP, lambdaP, lambdaH])*wo;
    Kw2 = wo*wo*eye(3);
    
    %Set the newly found gains in the simulation model
    set_param('model_for_simulation/Passive DP wave filter/obsK11', 'Gain', mat2str(K11));
    set_param('model_for_simulation/Passive DP wave filter/obsK12', 'Gain', mat2str(K12));
    set_param('model_for_simulation/Passive DP wave filter/obsK2', 'Gain', mat2str(K2));
    set_param('model_for_simulation/Passive DP wave filter/obsK4', 'Gain', mat2str(K4));
    set_param('model_for_simulation/Passive DP wave filter/obsK3', 'Gain', mat2str(K3));
    set_param('model_for_simulation/Passive DP wave filter/inv(T_b)', 'Gain', mat2str(T_b_inv));
    set_param('model_for_simulation/Passive DP wave filter/obsKw1', 'Gain', mat2str(Kw1));
    set_param('model_for_simulation/Passive DP wave filter/obsKw2', 'Gain', mat2str(Kw2));
    
end