clear
clc

%% Initialization
tsim = 40;

eta_initial = [0,0,0]';
nu_initial  = [0,0,0]';

%% External forces
%Scaling factor
gamma = 90;
%Load RAO needed for calculating wave forces
load('Helping scripts/CSAD_model.mat')
load('Helping scripts/CSAD_modelABC.mat')
%Set wave parameters
waves = 1; %1 on 0 off
Hs = 0.04;%10/gamma;
omega_p = 0.68*sqrt(gamma);
wave_dir = pi/2;%pi/2;%Eastgoing waves
%current
nu_c_NED = [0,0,0]';

wave_amp_vec = 0.05;
wave_freq_vec = 0.83*2*pi;
wave_dir_vec = 7/4*pi;

%% Initialize reference filter
w_d = diag([0.1 0.1 0.1]);
zeta = diag([1 1 1]);
v_max = 0.1;

%% Initialize observer
lambda = diag([0.01 0.01 0.08]);
wo = diag([omega_p omega_p omega_p]);
KpP = 600;
KpH = 23;
Ts = 50;
wc = 15.5*eye(3);%2*wo*eye(3);
T_b = diag([Ts Ts Ts])/sqrt(gamma);
T_b_inv = diag([1/T_b(1,1) 1/T_b(2,2) 1/T_b(3,3)]);
K11 = -2*(eye(3)-lambda)*diag([wc(1,1)/wo(1,1) wc(2,2)/wo(2,2) wc(3,3)/wo(3,3)]);
K12 = 2*wo*(eye(3)-lambda);
K2 = wc;
K4 = diag([KpP, KpP, KpH]);
K3 = 10*(K4/Ts);
Kw1 = 2*lambda*wo*eye(3);
Kw2 = wo*wo*eye(3);

set_param('model_for_simulation/Passive DP wave filter/inv(T_b)', 'Gain', mat2str(T_b_inv));
set_param('model_for_simulation/Passive DP wave filter/obsK11', 'Gain', mat2str(K11));
set_param('model_for_simulation/Passive DP wave filter/obsK12', 'Gain', mat2str(K12));
set_param('model_for_simulation/Passive DP wave filter/obsK2', 'Gain', mat2str(K2));
set_param('model_for_simulation/Passive DP wave filter/obsK4', 'Gain', mat2str(K4));
set_param('model_for_simulation/Passive DP wave filter/obsK3', 'Gain', mat2str(K3));
set_param('model_for_simulation/Passive DP wave filter/obsKw1', 'Gain', mat2str(Kw1));
set_param('model_for_simulation/Passive DP wave filter/obsKw2', 'Gain', mat2str(Kw2));
    

%% Initialising Control Plant model
X_u		= -5.35;		
X_uu 	=  0;		
X_uuu	= -19.6312;

Y_v		= -10.16;   
Y_vv	= -0.8647;
Y_vvv	= -681.1745;	

Y_r		= -7.25;
Y_rr	= -3.45;
Y_rrr   = 0;

N_v		= 0;		
N_vv	= -0.2088;
N_vvv = 0;

N_r		= -14.55;
N_rr	= -9.9597;
N_rrr	= -0.3101;

N_rv	= 0.08;	
N_vr	= 0.08;
Y_rv	= -0.805;			
Y_vr	= -0.845;

X_ud =     -10;
Y_vd =     -105;
Y_rd =     -0.525;
N_vd =     -0.157;	
N_rd =     -3.4950;

Nur  = Y_rd;
Nuv  = -(Y_vd-X_ud);
Yur  = X_ud;

x_g     = 0.0375;
m       = 127.92;
I_z     = 61.967;

u   = 0;
v   = 0;
r   = 0;

d_11 = - X_u - X_uu*abs(u) - X_uuu*u^2;
d_22 = - Y_v - Y_vv*abs(v) - Y_rv*abs(r) - Y_vvv*v^2;
d_23 = - Y_r - Y_rr*abs(r) - Y_vr*abs(v) - Y_rrr*r^2 - Yur*u;
d_32 = - N_v - N_vv*abs(v) - N_rv*abs(r) - N_vvv*v^2 - Nuv*u;
d_33 = - N_r - N_rr*abs(r) - N_vr*abs(v) - N_rrr*r^2 - Nur*u;

D    = [d_11  0     0
        0     d_22  d_23
        0     d_32  d_33];

CA=[0                            0          Y_vd*v+Y_rd*r;
    0                            0                -X_ud*u;
    -Y_rd*r-Y_vd*v               X_ud*u                0];

CRB=[0             0     -m*(x_g*r+v);
     0             0      m*u;
     m*(x_g*r+v) -m*u     0]; 

C_3DOF=CRB+CA;

M_RB_c=[m     0      0;
        0     m      m*x_g;
        0     m*x_g  I_z];

M_A_c=[-X_ud    0      0;
         0    -Y_vd  -Y_rd;
         0    -N_vd  -N_rd];     
    
M = M_RB_c + M_A_c;

Lx1 =  1.0678;   Ly1 =  0.0;
Lx2 =  0.9344;   Ly2 =  0.11;
Lx3 =  0.9344;   Ly3 = -0.11;
Lx4 = -1.1644;   Ly4 =  0.0;
Lx5 = -0.9911;   Ly5 = -0.1644;
Lx6 = -0.9911;   Ly6 =  0.1644;

K_e1 = 1.491*diag([1,1,1,1,1,1]);
alpha = [pi pi/4 -pi/4 0 5*pi/4 3*pi/4]';
L = [Lx1 norm([Lx2 Ly2]) norm([Lx3 Ly3]) norm([Lx4 Ly4]) norm([Lx5 Ly5]) norm([Lx6 Ly6])];
v1 = [0 atan(Ly2/Lx2) atan(Ly3/Lx3) 0 atan(Ly5/Lx5) atan(Ly6/Lx6)];

T_m = [cos(alpha)'; sin(alpha)'; cos(v1).*sin(alpha)'.*L];
T_m(3,5)=-T_m(3,5);
T_m(3,6)=-T_m(3,6);

T = T_m*K_e1;
%nedskalering av sway force og yaw moment
%slik at det passer med hva som er m�lt i lab
%(dette m� simuleres for alle u for � gi mening)
T(2,2) = 0.95*T(2,2);
T(2,3) = 0.95*T(2,3);
T(2,5) = 0.95*T(2,5);
T(2,6) = 0.95*T(2,6);
T(3,2) = 0.85*T(2,2);
T(3,3) = 0.85*T(2,3);
T(3,5) = -0.85*T(2,5);
T(3,6) = -0.85*T(2,6);

T_pinv_m = pinv(T);


% Run IMU simulation testing based on stored data for R/V Gunnerus in
% 4-corner maneuvers of various sea states
%
% Author:   Roger Skjetne
% Created:  17th November 2017
% Revisions: 
% ________________________________________________________________
%


%% Simulation parameters

% sample time [s]
Ts = 0.01;    % Sample time of data (data stored with Ts = 0.01 sec.)

%% Environmental parameters
Env.g 	= 9.80665;		% Constant of gravity

%% Sensor parameters
Sensor.NED.LatLon = [63.486056, 10.362799]*pi/180; % In Trondheimsfjorden, North of Munkholmen

Sensors.bias_enable     = 1; % 1 for including sensor bias models, 0 for not.
Sensors.noise_enable    = 1; % 1 for inlcuding sensor noise, 0 for not.

% Noise specification - based on ADIS 16364 at SKT'17 trials, dataset 17, IMU2.
Sensor.IMU.phi_theta.StD = [0.30, 0.25]*pi/180*0.01;  
Sensor.IMU.omega.StD     = [0.26, 0.27, 0.25]*pi/180*0.01;
Sensor.IMU.f.StD         = [0.1887, 0.05518, 0.08167]*0.01;
Sensor.GC.psi.StD        = [0.01]*pi/180*0.01;
Sensor.GPS.Thermal.StD   = [0.5 0.5 1.0]*0.01;     %meters white noise StD
Sensor.GPS.Multipath.StD = [0.1 0.1 0.25]*0.01;     %meters white noise StD
Sensor.GPS.Velocity.StD  = [0.05 0.05 0.1]*0.01;     %meters/sec white noise StD


%% Gyrocompass: [yaw] -> DOUBLE CHECK VALUES
Sensor.GC1.Noise_enable = Sensors.noise_enable;
Sensor.GC1.Bias_enable  = Sensors.bias_enable;
Sensor.GC1.Bias = 0;
Sensor.GC1.PSD  = 1*Ts*(Sensor.GC.psi.StD).^2;               % radians


%% GPS: position measurement [surge sway] -> DOUBLE CHECK VALUES!
Sensor.GPS.Noise_enable = Sensors.noise_enable;
Sensor.GPS.Bias_enable  = Sensors.bias_enable;
Sensor.GPS.Bias_atm = [0 0 0]';   % bias from the atmosphere
Sensor.GPS.PSD_multipath = 1.0*Ts*Sensor.GPS.Multipath.StD.^2;  % power
Sensor.GPS.PSD_thermal   =  1.0*Ts*Sensor.GPS.Thermal.StD.^2;     %meters white noise power
Sensor.GPS.Vel.PSD       =  1.0*Ts*Sensor.GPS.Velocity.StD.^2;     %meters/sec white noise power
Sensor.GPS.Vel.Bias      =  [0 0 0]';     %meters/sec white noise power


%% IMU sensors

% Values based on Kongsberg Maritime MRU 5+ and DPS 200 (www.km.kongsberg.com)
% measurement noise (Markov process)

radius =1;

arm1=[-0.244 0.184 0.082];
arm2=[-0.950 -0.13 0];
arm3=[0.74 0.13 0.082];
arm4=[0.45 -0.16 0.082];

% IMU1: Linear acceleration  [surge sway heave] and Angular rates [roll pitch yaw]
Sensor.IMU1.arm = radius*arm1;
Sensor.IMU1.W = IMUsensormtrx(Sensor.IMU1.arm);

Sensor.IMU1.Bias_enable = Sensors.bias_enable;
Sensor.IMU1.Noise_enable = Sensors.noise_enable;
Sensor.IMU1.BiasInit = 0.0*[0.05 -0.1 0.2]';   
Sensor.IMU1.PSD = 1.0*Ts*Sensor.IMU.f.StD.^2;  % m^2/s    Values based on ADIS 16364
Sensor.IMU1.BiasPSD = 1e-10*Sensor.IMU1.PSD;   

Sensor.Gyro1.Bias_enable = Sensors.bias_enable;
Sensor.Gyro1.Noise_enable = Sensors.noise_enable;
Sensor.Gyro1.BiasInit = 0.1*(pi/180)*[0.01 -0.02 0.015]';   % bias in rad/s
Sensor.Gyro1.PSD = 1.0*Ts*Sensor.IMU.omega.StD.^2;   % rates in rad/s   Values based on ADIS 16364
Sensor.Gyro1.BiasPSD = 1e-10*Sensor.Gyro1.PSD;   

Sensor.IMU2.arm = radius*arm2;
Sensor.IMU2.W = IMUsensormtrx(Sensor.IMU2.arm);

Sensor.IMU2.Bias_enable = Sensors.bias_enable;
Sensor.IMU2.Noise_enable = Sensors.noise_enable;
Sensor.IMU2.BiasInit = 0.0*[-0.1 -0.05 -0.3]';   
Sensor.IMU2.PSD = 1.1*Ts*Sensor.IMU.f.StD.^2;  % m^2/s    Values based on ADIS 16364
Sensor.IMU2.BiasPSD = 0.1*Sensor.IMU2.PSD/10;   

Sensor.Gyro2.Bias_enable = Sensors.bias_enable;
Sensor.Gyro2.Noise_enable = Sensors.noise_enable;
Sensor.Gyro2.BiasInit = 0.1*(pi/180)*[0.01 -0.02 0.015]';   % bias in rad/s
Sensor.Gyro2.PSD = 1.1*Ts*Sensor.IMU.omega.StD.^2;   % rates in rad/s   Values based on ADIS 16364
Sensor.Gyro2.BiasPSD = 1.0*Sensor.Gyro2.PSD/10;   

Sensor.IMU3.arm = radius*arm3;
Sensor.IMU3.W = IMUsensormtrx(Sensor.IMU3.arm);

Sensor.IMU3.Bias_enable = Sensors.bias_enable;
Sensor.IMU3.Noise_enable = Sensors.noise_enable;
Sensor.IMU3.BiasInit = 0.0*[0.1 -0.1 0.05]';   
Sensor.IMU3.PSD = 0.9*Ts*Sensor.IMU.f.StD.^2;  % m^2/s    Values based on ADIS 16364
Sensor.IMU3.BiasPSD = 0.1*Sensor.IMU3.PSD/10;   

Sensor.Gyro3.Bias_enable = Sensors.bias_enable;
Sensor.Gyro3.Noise_enable = Sensors.noise_enable;
Sensor.Gyro3.BiasInit = 0.1*(pi/180)*[0.01 -0.02 0.015]';   % bias in rad/s
Sensor.Gyro3.PSD = 0.9*Ts*Sensor.IMU.omega.StD.^2;   % rates in rad/s   Values based on ADIS 16364
Sensor.Gyro3.BiasPSD = 1.0*Sensor.Gyro3.PSD/10;   

Sensor.IMU4.arm = radius*arm4;
Sensor.IMU4.W = IMUsensormtrx(Sensor.IMU4.arm);

Sensor.IMU4.Bias_enable = Sensors.bias_enable;
Sensor.IMU4.Noise_enable = Sensors.noise_enable;
Sensor.IMU4.BiasInit = 0.0*[-0.05 0.15 -0.05]';   
Sensor.IMU4.PSD = 1.5*Ts*Sensor.IMU.f.StD.^2;  % m^2/s    Values based on ADIS 16364
Sensor.IMU4.BiasPSD = 0.1*Sensor.IMU4.PSD/10;   

Sensor.Gyro4.Bias_enable = Sensors.bias_enable;
Sensor.Gyro4.Noise_enable = Sensors.noise_enable;
Sensor.Gyro4.BiasInit = 0.1*(pi/180)*[0.01 -0.02 0.015]';   % bias in rad/s
Sensor.Gyro4.PSD = 1.5*Ts*Sensor.IMU.omega.StD.^2;   % rates in rad/s   Values based on ADIS 16364
Sensor.Gyro4.BiasPSD = 1.0*Sensor.Gyro4.PSD/10;   


% IMU combination parameters
Sensor.IMU.N = 4; % Number of IMUs
Sensor.IMU.W = [Sensor.IMU1.W; Sensor.IMU2.W; Sensor.IMU3.W; Sensor.IMU4.W];
Sensor.IMU.Winv = pinv(Sensor.IMU.W);
Ha_sel     = [eye(3), zeros(3,9)];
Halpha_sel = [zeros(3,3), eye(3), zeros(3,6)];
Sensor.IMU.Sel_linacc = Ha_sel*Sensor.IMU.Winv;
Sensor.IMU.Sel_angacc = Halpha_sel*Sensor.IMU.Winv;
